@@ -66,9 +66,14 @@ static const gchar *_quark_strings[] = {
   "GstMessageResetTime",
   "GstMessageToc", "GstEventTocGlobal", "GstEventTocCurrent",
   "GstEventSegmentDone",
-  "GstEventStreamStart", "stream-id", "GstEventContext", "GstQueryContext",
-  "GstMessageNeedContext", "GstMessageHaveContext", "context", "context-types",
-  "GstMessageStreamStart", "group-id", "uri-redirection"
+  "GstEventStreamStart", "stream-id", "GstQueryContext",
+  "GstMessageNeedContext", "GstMessageHaveContext", "context", "context-type",
+  "GstMessageStreamStart", "group-id", "uri-redirection",
+  "GstMessageDeviceAdded", "GstMessageDeviceRemoved", "device",
+  "uri-redirection-permanent", "GstMessagePropertyNotify", "property-name",
+  "property-value", "streams", "GstEventSelectStreams",
+  "GstMessageStreamCollection", "collection", "stream", "stream-collection",
+  "GstMessageStreamsSelected"
 };
 
 GQuark _priv_gst_quark_table[GST_QUARK_MAX];
