@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -165,6 +165,29 @@ gst_alloc_trace_print (const GstAllocTrace * trace)
 
         if (type == GST_TYPE_CAPS) {
           extra = gst_caps_to_string (data);
+        } else if (type == GST_TYPE_EVENT) {
+          const GstStructure *s = gst_event_get_structure (data);
+
+          if (s == NULL)
+            extra = g_strdup_printf ("%s", GST_EVENT_TYPE_NAME (data));
+          else
+            extra = gst_structure_to_string (s);
+        } else if (type == GST_TYPE_MESSAGE) {
+          const GstStructure *s = gst_message_get_structure (data);
+
+          if (s == NULL)
+            extra = g_strdup_printf ("%s", GST_MESSAGE_TYPE_NAME (data));
+          else
+            extra = gst_structure_to_string (s);
+        } else if (type == GST_TYPE_BUFFER) {
+          guint size = gst_buffer_get_size (data);
+
+          extra = g_strdup_printf ("%u bytes", size);
+        } else if (type == GST_TYPE_MEMORY) {
+          GstMemory *mem = (GstMemory *) data;
+
+          extra = g_strdup_printf ("%u bytes, %s allocator", (guint) mem->size,
+              mem->allocator ? mem->allocator->mem_type : "unknown");
         }
         refcount = GST_MINI_OBJECT_REFCOUNT_VALUE (data);
       }
